/* Copyright 2017-2021 Xaptum, Inc.
 * Copyright 2021 NXP
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Functions in this file comes from Xaptum, https://github.com/xaptum/ecdaa
 * */

/* ************************************************************************** */
/* Includes                                                                   */
/* ************************************************************************** */
#include <stdio.h>
#include "nxEnsure.h"
#include <nxLog_App.h>

#include "ex_sss_fido_hash.h"
#include "ex_sss_fido_big_256_28.h"

/* clang-format off */

/* ************************************************************************** */
/* Local Defines                                                              */
/* ************************************************************************** */

/* ************************************************************************** */
/* Structures and Typedefs                                                    */
/* ************************************************************************** */

/* ************************************************************************** */
/* Global Variables                                                           */
/* ************************************************************************** */


/* ************************************************************************** */
/* Static function declarations                                               */
/* ************************************************************************** */
static void explicit_bzero(void *const pnt, const size_t len);
static void convert_hash_to_big_256_28(BIG_256_28 *big_out, hash256 *hash);
static inline chunk muladd_256_28(chunk x,chunk y,chunk c,chunk *r);


/* ************************************************************************** */
/* Private Functions                                                          */
/* ************************************************************************** */

/* ************************************************************************** */
/* Public Functions                                                           */
/* ************************************************************************** */

static void explicit_bzero(void *const pnt, const size_t len)
{
    volatile unsigned char * pnt_ =
        (volatile unsigned char *) pnt;

    size_t i = (size_t) 0U;
    while (i < len) {
        pnt_[i++] = 0U;
    }
}

static void convert_hash_to_big_256_28(BIG_256_28 *big_out, hash256 *hash)
{
    char hash_as_bytes[32] = {0};
    ENSURE_OR_RETURN(hash->hlen == sizeof(hash_as_bytes));

    // Clears the hash object after output.
    HASH256_hash(hash, hash_as_bytes);

    // Convert byte-string to un-normalized BIG.
    BIG_256_28_fromBytesLen(*big_out, hash_as_bytes, hash->hlen);

    explicit_bzero(hash_as_bytes, sizeof(hash_as_bytes));
}

void BIG_256_28_rawoutput(BIG_256_28 a)
{
    int i;
    printf("(");
    for (i=0; i<NLEN_256_28-1; i++){
        printf("%x,",(unsigned int) a[i]);
    }
    printf("%x)",(unsigned int) a[NLEN_256_28-1]);
    printf("\n");
}

/* Set a=0 */
void BIG_256_28_zero(BIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        a[i]=0;
}

void BIG_256_28_fromBytes(BIG_256_28 a,char *b)
{
    int i;
    BIG_256_28_zero(a);
    for (i=0; i<MODBYTES_256_28; i++)
    {
        BIG_256_28_fshl(a,8);
        a[0]+=(int)(unsigned char)b[i];
        //BIG_256_28_inc(a,(int)(unsigned char)b[i]); BIG_256_28_norm(a);
    }
}

void BIG_256_28_fromBytesLen(BIG_256_28 a,char *b,int s)
{
    int i,len=s;
    BIG_256_28_zero(a);

    if (len>MODBYTES_256_28) len=MODBYTES_256_28;
    for (i=0; i<len; i++)
    {
        BIG_256_28_fshl(a,8);
        a[0]+=(int)(unsigned char)b[i];
    }
}

/* Fast shift left of a by n bits, where n less than a word, Return excess (but store it as well) */
/* a MUST be normalised */
/* SU= 16 */
int BIG_256_28_fshl(BIG_256_28 a,int n)
{
    int i;

    a[NLEN_256_28-1]=((a[NLEN_256_28-1]<<n))|(a[NLEN_256_28-2]>>(BASEBITS_256_28-n)); /* top word not masked */
    for (i=NLEN_256_28-2; i>0; i--)
        a[i]=((a[i]<<n)&BMASK_256_28)|(a[i-1]>>(BASEBITS_256_28-n));
    a[0]=(a[0]<<n)&BMASK_256_28;

    return (int)(a[NLEN_256_28-1]>>((8*MODBYTES_256_28)%BASEBITS_256_28)); /* return excess - only used in ff.c */
}


/* you gotta keep the sign of carry! Look - no branching! */
/* Note that sign bit is needed to disambiguate between +ve and -ve values */
/* normalise BIG - force all digits < 2^BASEBITS */
chunk BIG_256_28_norm(BIG_256_28 a)
{
    int i;
    chunk d,carry=0;
    for (i=0; i<NLEN_256_28-1; i++)
    {
        d=a[i]+carry;
        a[i]=d&BMASK_256_28;
        carry=d>>BASEBITS_256_28;
    }
    a[NLEN_256_28-1]=(a[NLEN_256_28-1]+carry);

    return (a[NLEN_256_28-1]>>((8*MODBYTES_256_28)%BASEBITS_256_28));  /* only used in ff.c */
}

/* Copy b=a */
void BIG_256_28_copy(BIG_256_28 b,BIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        b[i]=a[i];
}

/* Faster shift right of a by k bits. Return shifted out part */
/* a MUST be normalised */
/* SU= 16 */
int BIG_256_28_fshr(BIG_256_28 a,int k)
{
    int i;
    chunk r=a[0]&(((chunk)1<<k)-1); /* shifted out part */
    for (i=0; i<NLEN_256_28-1; i++)
        a[i]=(a[i]>>k)|((a[i+1]<<(BASEBITS_256_28-k))&BMASK_256_28);
    a[NLEN_256_28-1]=a[NLEN_256_28-1]>>k;
    return (int)r;
}

/* convert BIG to/from bytes */
/* SU= 64 */
void BIG_256_28_toBytes(char *b,BIG_256_28 a)
{
    int i;
    BIG_256_28 c;
    BIG_256_28_norm(a);
    BIG_256_28_copy(c,a);
    for (i=MODBYTES_256_28-1; i>=0; i--)
    {
        b[i]=c[0]&0xff;
        BIG_256_28_fshr(c,8);
    }
}

void BIG_256_28_toBuffer(uint8_t *b, uint32_t * valid_len, BIG_256_28 a)
{
    int i, j = 0, found_no_zero = 0;
    chunk * src = a;

    if ((b != NULL) && (src != NULL)){
        for (i=NLEN_256_28-1; i>=0; i--)
        {
            if ((src[i] == 0) && (found_no_zero == 0)){
                continue;
            }
            else if (src[i] != 0){
                found_no_zero = 1;
            }

            b[j++] = (uint8_t)((src[i] >> 3 * 8) & 0xFF);
            b[j++] = (uint8_t)((src[i] >> 2 * 8) & 0xFF);
            b[j++] = (uint8_t)((src[i] >> 1 * 8) & 0xFF);
            b[j++] = (uint8_t)((src[i] >> 0 * 8) & 0xFF);
        }
    }

    *valid_len = j;
}

/* Compare a and b. Return 1 for a>b, -1 for a<b, 0 for a==b */
/* a and b MUST be normalised before call */
int BIG_256_28_comp(BIG_256_28 a,BIG_256_28 b)
{
    int i;
    for (i=NLEN_256_28-1; i>=0; i--)
    {
        if (a[i]==b[i]) continue;
        if (a[i]>b[i]) return 1;
        else  return -1;
    }
    return 0;
}

/* Set c=a-b */
/* SU= 8 */
void BIG_256_28_sub(BIG_256_28 c,BIG_256_28 a,BIG_256_28 b)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        c[i]=a[i]-b[i];
}

/* Move b to a if d=1 */
void BIG_256_28_cmove(BIG_256_28 f,BIG_256_28 g,int d)
{
    int i;
    chunk b=(chunk)-d;

    for (i=0; i<NLEN_256_28; i++)
    {
        f[i]^=(f[i]^g[i])&b;
    }
}

/* Set b=b mod c */
/* SU= 16 */
void BIG_256_28_mod(BIG_256_28 b,BIG_256_28 c)
{
    int k=0;
    BIG_256_28 r; /**/

    BIG_256_28_norm(b);
    if (BIG_256_28_comp(b,c)<0)
        return;
    do
    {
        BIG_256_28_fshl(c,1);
        k++;
    }
    while (BIG_256_28_comp(b,c)>=0);

    while (k>0)
    {
        BIG_256_28_fshr(c,1);

// constant time...
        BIG_256_28_sub(r,b,c);
        BIG_256_28_norm(r);
        BIG_256_28_cmove(b,r,1-((r[NLEN_256_28-1]>>(CHUNK-1))&1));
        k--;
    }
}

/* b=a */
void BIG_256_28_rcopy(BIG_256_28 b,const BIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        b[i]=a[i];
}

/* test a=0? */
int BIG_256_28_iszilch(BIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        if (a[i]!=0) return 0;
    return 1;
}

/* test a=1? */
int BIG_256_28_isunity(BIG_256_28 a)
{
    int i;
    for(i=1; i<NLEN_256_28; i++)
        if (a[i]!=0) return 0;
    if (a[0]!=1) return 0;
    return 1;
}

/* set a=1 */
void BIG_256_28_one(BIG_256_28 a)
{
    int i;
    a[0]=1;
    for (i=1; i<NLEN_256_28; i++)
        a[i]=0;
}

/* Set c=a*b */
/* SU= 72 */
void BIG_256_28_mul(DBIG_256_28 c,BIG_256_28 a,BIG_256_28 b)
{
    int i;
    dchunk t,co;
    dchunk s;
    dchunk d[NLEN_256_28];
    int k;

    /* Faster to Combafy it.. Let the compiler unroll the loops! */

    /* faster psuedo-Karatsuba method */
    for (i=0; i<NLEN_256_28; i++)
        d[i]=(dchunk)a[i]*b[i];

    s=d[0];
    t=s;
    c[0]=(chunk)t&BMASK_256_28;
    co=t>>BASEBITS_256_28;

    for (k=1; k<NLEN_256_28; k++)
    {
        s+=d[k];
        t=co+s;
        for (i=k; i>=1+k/2; i--) t+=(dchunk)(a[i]-a[k-i])*(b[k-i]-b[i]);
        c[k]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
    }
    for (k=NLEN_256_28; k<2*NLEN_256_28-1; k++)
    {
        s-=d[k-NLEN_256_28];
        t=co+s;
        for (i=NLEN_256_28-1; i>=1+k/2; i--) t+=(dchunk)(a[i]-a[k-i])*(b[k-i]-b[i]);
        c[k]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
    }
    c[2*NLEN_256_28-1]=(chunk)co;
}

/* Montgomery reduction */
void BIG_256_28_monty(BIG_256_28 a,BIG_256_28 md,chunk MC,DBIG_256_28 d)
{
    int i,k;

    dchunk t,c,s;
    dchunk dd[NLEN_256_28];
    chunk v[NLEN_256_28];

    t=d[0];
    v[0]=((chunk)t*MC)&BMASK_256_28;
    t+=(dchunk)v[0]*md[0];
    c=(t>>BASEBITS_256_28)+d[1];
    s=0;

    for (k=1; k<NLEN_256_28; k++)
    {
        t=c+s+(dchunk)v[0]*md[k];
        for (i=k-1; i>k/2; i--) t+=(dchunk)(v[k-i]-v[i])*(md[i]-md[k-i]);
        v[k]=((chunk)t*MC)&BMASK_256_28;
        t+=(dchunk)v[k]*md[0];
        c=(t>>BASEBITS_256_28)+d[k+1];
        dd[k]=(dchunk)v[k]*md[k];
        s+=dd[k];
    }
    for (k=NLEN_256_28; k<2*NLEN_256_28-1; k++)
    {
        t=c+s;
        for (i=NLEN_256_28-1; i>=1+k/2; i--) t+=(dchunk)(v[k-i]-v[i])*(md[i]-md[k-i]);
        a[k-NLEN_256_28]=(chunk)t&BMASK_256_28;
        c=(t>>BASEBITS_256_28)+d[k+1];
        s-=dd[k-NLEN_256_28+1];
    }
    a[NLEN_256_28-1]=(chunk)c&BMASK_256_28;

}

/* return LSB of a */
int BIG_256_28_parity(BIG_256_28 a)
{
    return a[0]%2;
}

/* General shift rightof a by k bits */
/* a MUST be normalised */
/* SU= 32 */
void BIG_256_28_shr0(BIG_256_28 a,int k)
{
    int i;
    int n=k%BASEBITS_256_28;
    int m=k/BASEBITS_256_28;
    for (i=0; i<NLEN_256_28-m-1; i++)
        a[i]=(a[m+i]>>n)|((a[m+i+1]<<(BASEBITS_256_28-n))&BMASK_256_28);
    if (NLEN_256_28>m)  a[NLEN_256_28-m-1]=a[NLEN_256_28-1]>>n;
    for (i=NLEN_256_28-m; i<NLEN_256_28; i++) a[i]=0;

}

void BIG_256_28_shr(BIG_256_28 a,int k)
{
    int i=BASEBITS_256_28;
    int n=k%BASEBITS_256_28;
    int m=k/BASEBITS_256_28;
    for (i=0; i<NLEN_256_28-m-1; i++) {
        a[i]=(a[m+i]>>n)|((a[m+i+1]<<(BASEBITS_256_28-n))&BMASK_256_28);
    }
    if (NLEN_256_28>m)  a[NLEN_256_28-m-1]=a[NLEN_256_28-1]>>n;
    for (i=NLEN_256_28-m; i<NLEN_256_28; i++) a[i]=0;

}


/* Set c=a+b */
/* SU= 8 */
void BIG_256_28_add(BIG_256_28 c,BIG_256_28 a,BIG_256_28 b)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        c[i]=a[i]+b[i];
}

/* Set c=a*a */
/* SU= 80 */
void BIG_256_28_sqr(DBIG_256_28 c,BIG_256_28 a)
{
    int i,j;
    dchunk t,co;

    /* Note 2*a[i] in loop below and extra addition */
    t=(dchunk)a[0]*a[0];
    c[0]=(chunk)t&BMASK_256_28;
    co=t>>BASEBITS_256_28;

    for (j=1; j<NLEN_256_28-1; )
    {
        t=(dchunk)a[j]*a[0];
        for (i=1; i<(j+1)/2; i++)
        {
            t+=(dchunk)a[j-i]*a[i];
        }
        t+=t;
        t+=co;
        c[j]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
        j++;
        t=(dchunk)a[j]*a[0];
        for (i=1; i<(j+1)/2; i++)
        {
            t+=(dchunk)a[j-i]*a[i];
        }
        t+=t;
        t+=co;
        t+=(dchunk)a[j/2]*a[j/2];
        c[j]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
        j++;
    }

    for (j=NLEN_256_28-1+NLEN_256_28%2; j<DNLEN_256_28-3; )
    {
        t=(dchunk)a[NLEN_256_28-1]*a[j-NLEN_256_28+1];
        for (i=j-NLEN_256_28+2; i<(j+1)/2; i++)
        {
            t+=(dchunk)a[j-i]*a[i];
        }
        t+=t;
        t+=co;
        c[j]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
        j++;
        t=(dchunk)a[NLEN_256_28-1]*a[j-NLEN_256_28+1];
        for (i=j-NLEN_256_28+2; i<(j+1)/2; i++)
        {
            t+=(dchunk)a[j-i]*a[i];
        }
        t+=t;
        t+=co;
        t+=(dchunk)a[j/2]*a[j/2];
        c[j]=(chunk)t&BMASK_256_28;
        co=t>>BASEBITS_256_28;
        j++;
    }

    t=(dchunk)a[NLEN_256_28-2]*a[NLEN_256_28-1];
    t+=t;
    t+=co;
    c[DNLEN_256_28-3]=(chunk)t&BMASK_256_28;
    co=t>>BASEBITS_256_28;

    t=(dchunk)a[NLEN_256_28-1]*a[NLEN_256_28-1]+co;
    c[DNLEN_256_28-2]=(chunk)t&BMASK_256_28;
    co=t>>BASEBITS_256_28;
    c[DNLEN_256_28-1]=(chunk)co;
}

/* Set r=1/a mod p. Binary method */
/* SU= 240 */
void BIG_256_28_invmodp(BIG_256_28 r,BIG_256_28 a,BIG_256_28 p)
{
    BIG_256_28 u,v,x1,x2,t,one;
    BIG_256_28_mod(a,p);
    BIG_256_28_copy(u,a);
    BIG_256_28_copy(v,p);
    BIG_256_28_one(one);
    BIG_256_28_copy(x1,one);
    BIG_256_28_zero(x2);

    while (BIG_256_28_comp(u,one)!=0 && BIG_256_28_comp(v,one)!=0)
    {
        while (BIG_256_28_parity(u)==0)
        {
            BIG_256_28_shr(u,1);
            if (BIG_256_28_parity(x1)!=0)
            {
                BIG_256_28_add(x1,p,x1);
                BIG_256_28_norm(x1);
            }
            BIG_256_28_shr(x1,1);
        }
        while (BIG_256_28_parity(v)==0)
        {
            BIG_256_28_shr(v,1);
            if (BIG_256_28_parity(x2)!=0)
            {
                BIG_256_28_add(x2,p,x2);
                BIG_256_28_norm(x2);
            }
            BIG_256_28_shr(x2,1);
        }
        if (BIG_256_28_comp(u,v)>=0)
        {
            BIG_256_28_sub(u,u,v);
            BIG_256_28_norm(u);
            if (BIG_256_28_comp(x1,x2)>=0) BIG_256_28_sub(x1,x1,x2);
            else
            {
                BIG_256_28_sub(t,p,x2);
                BIG_256_28_add(x1,x1,t);
            }
            BIG_256_28_norm(x1);
        }
        else
        {
            BIG_256_28_sub(v,v,u);
            BIG_256_28_norm(v);
            if (BIG_256_28_comp(x2,x1)>=0) BIG_256_28_sub(x2,x2,x1);
            else
            {
                BIG_256_28_sub(t,p,x1);
                BIG_256_28_add(x2,x2,t);
            }
            BIG_256_28_norm(x2);
        }
    }
    if (BIG_256_28_comp(u,one)==0)
        BIG_256_28_copy(r,x1);
    else
        BIG_256_28_copy(r,x2);
}

void BIG_256_28_dzero(DBIG_256_28 a)
{
    int i;
    for (i=0; i<DNLEN_256_28; i++)
        a[i]=0;
}

/* Copy BIG to bottom half of DBIG */
void BIG_256_28_dscopy(DBIG_256_28 b,BIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28-1; i++)
        b[i]=a[i];

    b[NLEN_256_28-1]=a[NLEN_256_28-1]&BMASK_256_28; /* top word normalized */
    b[NLEN_256_28]=a[NLEN_256_28-1]>>BASEBITS_256_28;

    for (i=NLEN_256_28+1; i<DNLEN_256_28; i++) b[i]=0;
}

/* Set c=c+d */
void BIG_256_28_inc(BIG_256_28 c,int d)
{
    BIG_256_28_norm(c);
    c[0]+=(chunk)d;
}

/* return number of bits in a */
/* SU= 8 */
int BIG_256_28_nbits(BIG_256_28 a)
{
    int bts,k=NLEN_256_28-1;
    chunk c;
    BIG_256_28_norm(a);
    while (k>=0 && a[k]==0) k--;
    if (k<0) return 0;
    bts=BASEBITS_256_28*k;
    c=a[k];
    while (c!=0)
    {
        c/=2;
        bts++;
    }
    return bts;
}

/* return last n bits of a, where n is small < BASEBITS */
/* SU= 16 */
int BIG_256_28_lastbits(BIG_256_28 a,int n)
{
    int msk=(1<<n)-1;
    BIG_256_28_norm(a);
    return ((int)a[0])&msk;
}

/* Method required to calculate x*y+c+r, bottom half in r, top half returned */
static inline chunk muladd_256_28(chunk x,chunk y,chunk c,chunk *r)
{
    dchunk prod=(dchunk)x*y+c+*r;
    *r=(chunk)prod&BMASK_256_28;
    return (chunk)(prod>>BASEBITS_256_28);
}

/* multiplication r=a*c by larger integer - c<=FEXCESS */
/* SU= 24 */
chunk BIG_256_28_pmul(BIG_256_28 r,BIG_256_28 a,int c)
{
    int i;
    chunk ak,carry=0;
//    BIG_256_28_norm(a);
    for (i=0; i<NLEN_256_28; i++)
    {
        ak=a[i];
        r[i]=0;
        carry=muladd_256_28(ak,(chunk)c,carry,&r[i]);
    }
    return carry;
}

/* Set c=c-1 */
void BIG_256_28_dec(BIG_256_28 c,int d)
{
    BIG_256_28_norm(c);
    c[0]-=(chunk)d;
}

/* r/=3 */
int BIG_256_28_div3(BIG_256_28 r)
{
    int i;
    chunk ak,base,carry=0;
    BIG_256_28_norm(r);
    base=((chunk)1<<BASEBITS_256_28);
    for (i=NLEN_256_28-1; i>=0; i--)
    {
        ak=(carry*base+r[i]);
        r[i]=ak/3;
        carry=ak%3;
    }
    return (int)carry;
}

/* Set r=a*b mod m */
/* SU= 96 */
void BIG_256_28_modmul(BIG_256_28 r,BIG_256_28 a,BIG_256_28 b,BIG_256_28 m)
{
    DBIG_256_28 d;
    BIG_256_28_mod(a,m);
    BIG_256_28_mod(b,m);

    BIG_256_28_mul(d,a,b);
    BIG_256_28_dmod(r,d,m);
}

/* Set a=b mod c, b is destroyed. Slow but rarely used. */
/* SU= 96 */
void BIG_256_28_dmod(BIG_256_28 a,DBIG_256_28 b,BIG_256_28 c)
{
    int k=0;
    DBIG_256_28 m,r;
    BIG_256_28_dnorm(b);
    BIG_256_28_dscopy(m,c);

    if (BIG_256_28_dcomp(b,m)<0)
    {
        BIG_256_28_sdcopy(a,b);
        return;
    }

    do
    {
        BIG_256_28_dshl(m,1);
        k++;
    }
    while (BIG_256_28_dcomp(b,m)>=0);

    while (k>0)
    {
        BIG_256_28_dshr(m,1);
// constant time...
        BIG_256_28_dsub(r,b,m);
        BIG_256_28_dnorm(r);
        BIG_256_28_dcmove(b,r,1-((r[DNLEN_256_28-1]>>(CHUNK-1))&1));

        k--;
    }
    BIG_256_28_sdcopy(a,b);
}

void BIG_256_28_dnorm(DBIG_256_28 a)
{
    int i;
    chunk d,carry=0;
    for (i=0; i<DNLEN_256_28-1; i++)
    {
        d=a[i]+carry;
        a[i]=d&BMASK_256_28;
        carry=d>>BASEBITS_256_28;
    }
    a[DNLEN_256_28-1]=(a[DNLEN_256_28-1]+carry);
}

int BIG_256_28_dcomp(DBIG_256_28 a,DBIG_256_28 b)
{
    int i;
    for (i=DNLEN_256_28-1; i>=0; i--)
    {
        if (a[i]==b[i]) continue;
        if (a[i]>b[i]) return 1;
        else  return -1;
    }
    return 0;
}

/* Copy bottom half of DBIG to BIG */
void BIG_256_28_sdcopy(BIG_256_28 b,DBIG_256_28 a)
{
    int i;
    for (i=0; i<NLEN_256_28; i++)
        b[i]=a[i];
}

/* double length left shift of a by k bits - k can be > BASEBITS , a MUST be normalised */
/* SU= 32 */
void BIG_256_28_dshl(DBIG_256_28 a,int k)
{
    int i;
    int n=k%BASEBITS_256_28;
    int m=k/BASEBITS_256_28;

    a[DNLEN_256_28-1]=((a[DNLEN_256_28-1-m]<<n))|(a[DNLEN_256_28-m-2]>>(BASEBITS_256_28-n));

    for (i=DNLEN_256_28-2; i>m; i--)
        a[i]=((a[i-m]<<n)&BMASK_256_28)|(a[i-m-1]>>(BASEBITS_256_28-n));
    a[m]=(a[0]<<n)&BMASK_256_28;
    for (i=0; i<m; i++) a[i]=0;

}

/* double length right shift of a by k bits - can be > BASEBITS */
/* SU= 32 */
void BIG_256_28_dshr(DBIG_256_28 a,int k)
{
    int i;
    int n=k%BASEBITS_256_28;
    int m=k/BASEBITS_256_28;
    for (i=0; i<DNLEN_256_28-m-1; i++)
        a[i]=(a[m+i]>>n)|((a[m+i+1]<<(BASEBITS_256_28-n))&BMASK_256_28);
    a[DNLEN_256_28-m-1]=a[DNLEN_256_28-1]>>n;
    for (i=DNLEN_256_28-m; i<DNLEN_256_28; i++ ) a[i]=0;
}

/* SU= 8 */

void BIG_256_28_dsub(DBIG_256_28 c,DBIG_256_28 a,DBIG_256_28 b)
{
    int i;
    for (i=0; i<DNLEN_256_28; i++)
        c[i]=a[i]-b[i];
}

/* Move g to f if d=1 */
void BIG_256_28_dcmove(DBIG_256_28 f,DBIG_256_28 g,int d)
{
    int i;
    chunk b=(chunk)-d;

    for (i=0; i<DNLEN_256_28; i++)
    {
        f[i]^=(f[i]^g[i])&b;
    }
}

/* Set a=a/b mod m */
/* SU= 136 */
void BIG_256_28_moddiv(BIG_256_28 r,BIG_256_28 a,BIG_256_28 b,BIG_256_28 m)
{
    DBIG_256_28 d;
    BIG_256_28 z;
    BIG_256_28_mod(a,m);
    BIG_256_28_invmodp(z,b,m);

    BIG_256_28_mul(d,a,z);
    BIG_256_28_dmod(r,d,m);
}

/* SU= 136 */

void BIG_256_28_sdiv(BIG_256_28 a,BIG_256_28 c)
{
    int d,k=0;
    BIG_256_28 m,e,b,r;
    BIG_256_28_norm(a);
    BIG_256_28_copy(b,a);
    BIG_256_28_copy(m,c);

    BIG_256_28_zero(a);
    BIG_256_28_zero(e);
    BIG_256_28_inc(e,1);

    while (BIG_256_28_comp(b,m)>=0)
    {
        BIG_256_28_fshl(e,1);
        BIG_256_28_fshl(m,1);
        k++;
    }

    while (k>0)
    {
        BIG_256_28_fshr(m,1);
        BIG_256_28_fshr(e,1);

        BIG_256_28_sub(r,b,m);
        BIG_256_28_norm(r);
        d=1-((r[NLEN_256_28-1]>>(CHUNK-1))&1);
        BIG_256_28_cmove(b,r,d);

        BIG_256_28_add(r,a,e);
        BIG_256_28_norm(r);
        BIG_256_28_cmove(a,r,d);
        k--;
    }
}

void big_256_28_from_hash(BIG_256_28 *big_out,
                       const uint8_t *msg_in,
                       uint32_t msg_len)
{
    hash256 hash;
    HASH256_init(&hash);

    // Process input one-byte-at-a-time.
    for (uint32_t i=0; i < msg_len; ++i) {
        HASH256_process(&hash, msg_in[i]);
    }

    convert_hash_to_big_256_28(big_out, &hash);
}

/* clang-format on */