/* Copyright 2017-2021 Xaptum, Inc.
 * Copyright 2021 NXP
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Functions in this file comes from Xaptum, https://github.com/xaptum/ecdaa
 * */

#ifndef EX_SSS_FIDO_BIG_256_28_H_
#define EX_SSS_FIDO_BIG_256_28_H_
/* ************************************************************************** */
/* Includes                                                                   */
/* ************************************************************************** */

/* clang-format off */

/*******************************************************************
* Defines
*******************************************************************/
#define MODBYTES_256_28 32  	/**< Number of bytes in Modulus */
#define BASEBITS_256_28 28 	/**< Numbers represented to base 2*BASEBITS */

#define CHUNK 32

#define NLEN_256_28 (1+((8*MODBYTES_256_28-1)/BASEBITS_256_28)) /**< length in bytes */
#define DNLEN_256_28 2*NLEN_256_28 /**< Double length in bytes */
#define BMASK_256_28 (((chunk)1<<BASEBITS_256_28)-1) /**< Mask = 2^BASEBITS-1 */

#define chunk int32_t		/**< C type corresponding to word length */
#define dchunk int64_t		/**< Always define double length chunk type if available */

typedef chunk BIG_256_28[NLEN_256_28];     /**< Define type BIG as array of chunks */
typedef chunk DBIG_256_28[DNLEN_256_28];   /**< Define type DBIG as array of chunks */

/*******************************************************************************
 * Prototypes
 ******************************************************************************/

void BIG_256_28_rawoutput(BIG_256_28 a);
void BIG_256_28_zero(BIG_256_28 a);
void BIG_256_28_fromBytes(BIG_256_28 a,char *b);
void BIG_256_28_fromBytesLen(BIG_256_28 a,char *b,int s);
int BIG_256_28_fshl(BIG_256_28 a,int n);
chunk BIG_256_28_norm(BIG_256_28 a);
void BIG_256_28_copy(BIG_256_28 b,BIG_256_28 a);
int BIG_256_28_fshr(BIG_256_28 a,int k);
void BIG_256_28_toBytes(char *b,BIG_256_28 a);
void BIG_256_28_toBuffer(uint8_t *b, uint32_t * valid_len, BIG_256_28 a);
int BIG_256_28_comp(BIG_256_28 a,BIG_256_28 b);
void BIG_256_28_sub(BIG_256_28 c,BIG_256_28 a,BIG_256_28 b);
void BIG_256_28_cmove(BIG_256_28 f,BIG_256_28 g,int d);
void BIG_256_28_mod(BIG_256_28 b,BIG_256_28 c);
void BIG_256_28_rcopy(BIG_256_28 b,const BIG_256_28 a);
int BIG_256_28_iszilch(BIG_256_28 a);
int BIG_256_28_isunity(BIG_256_28 a);
void BIG_256_28_one(BIG_256_28 a);
void BIG_256_28_mul(DBIG_256_28 c,BIG_256_28 a,BIG_256_28 b);
void BIG_256_28_monty(BIG_256_28 a,BIG_256_28 md,chunk MC,DBIG_256_28 d);
int BIG_256_28_parity(BIG_256_28 a);
void BIG_256_28_shr(BIG_256_28 a,int k);
void BIG_256_28_add(BIG_256_28 c,BIG_256_28 a,BIG_256_28 b);
void BIG_256_28_sqr(DBIG_256_28 c,BIG_256_28 a);
void BIG_256_28_invmodp(BIG_256_28 r,BIG_256_28 a,BIG_256_28 p);
void BIG_256_28_dzero(DBIG_256_28 a);
void BIG_256_28_dscopy(DBIG_256_28 b,BIG_256_28 a);
void BIG_256_28_inc(BIG_256_28 c,int d);
int BIG_256_28_nbits(BIG_256_28 a);
int BIG_256_28_lastbits(BIG_256_28 a,int n);
chunk BIG_256_28_pmul(BIG_256_28 r,BIG_256_28 a,int c);
void BIG_256_28_dec(BIG_256_28 c,int d);
int BIG_256_28_div3(BIG_256_28 r);
void BIG_256_28_modmul(BIG_256_28 r,BIG_256_28 a,BIG_256_28 b,BIG_256_28 m);
void BIG_256_28_dmod(BIG_256_28 a,DBIG_256_28 b,BIG_256_28 c);
void BIG_256_28_dnorm(DBIG_256_28 a);
int BIG_256_28_dcomp(DBIG_256_28 a,DBIG_256_28 b);
void BIG_256_28_sdcopy(BIG_256_28 b,DBIG_256_28 a);
void BIG_256_28_dshl(DBIG_256_28 a,int k);
void BIG_256_28_dshr(DBIG_256_28 a,int k);
void BIG_256_28_dsub(DBIG_256_28 c,DBIG_256_28 a,DBIG_256_28 b);
void BIG_256_28_dcmove(DBIG_256_28 f,DBIG_256_28 g,int d);
void BIG_256_28_moddiv(BIG_256_28 r,BIG_256_28 a,BIG_256_28 b,BIG_256_28 m);
void BIG_256_28_sdiv(BIG_256_28 a,BIG_256_28 c);


void big_256_28_from_hash(BIG_256_28 *big_out,
                       const uint8_t *msg_in,
                       uint32_t msg_len);

/* clang-format on */

#endif /*  EX_SSS_FIDO_BIG_256_28_H_ */
