/* Copyright 2017-2021 Xaptum, Inc.
 * Copyright 2021 NXP
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Functions in this file comes from Xaptum, https://github.com/xaptum/ecdaa
 * */

#ifndef EX_SSS_FIDO_ECP_H_
#define EX_SSS_FIDO_ECP_H_
/* ************************************************************************** */
/* Includes                                                                   */
/* ************************************************************************** */

/* clang-format off */

/*******************************************************************
* Defines
*******************************************************************/

typedef struct
{
    BIG_256_28 g;	/**< Big representation of field element */
    int32_t XES;	/**< Excess */
} FP_FP256BN;

/**
	@brief ECP structure - Elliptic Curve Point over base field
*/

typedef struct
{
    int inf; /**< Infinity Flag - not needed for Edwards representation */

    FP_FP256BN x; /**< x-coordinate of point */
    FP_FP256BN y; /**< y-coordinate of point. Not needed for Montgomery representation */
    FP_FP256BN z;/**< z-coordinate of point */
} ECP_FP256BN;

typedef struct
{
    int len;   /**< length in bytes  */
    int max;   /**< max length allowed - enforce truncation  */
    char *val; /**< byte array  */
} octet;

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
void ECP_FP256BN_copy(ECP_FP256BN *P,ECP_FP256BN *Q);
void ECP_FP256BN_mul(ECP_FP256BN *P,BIG_256_28 e);
void ECP_FP256BN_cmove(ECP_FP256BN *P,ECP_FP256BN *Q,int d);
void ECP_FP256BN_inf(ECP_FP256BN *P);
int ECP_FP256BN_isinf(ECP_FP256BN *P);
void ECP_FP256BN_affine(ECP_FP256BN *P);
int ECP_FP256BN_get(BIG_256_28 x,BIG_256_28 y,ECP_FP256BN *P);
int ECP_FP256BN_set(ECP_FP256BN *P,BIG_256_28 x,BIG_256_28 y);
void ECP_FP256BN_rhs(FP_FP256BN *v,FP_FP256BN *x);
void ECP_FP256BN_dbl(ECP_FP256BN *P);
void ECP_FP256BN_add(ECP_FP256BN *P,ECP_FP256BN *Q);
void ECP_FP256BN_select(ECP_FP256BN *P,ECP_FP256BN W[],int32_t b);
void ECP_FP256BN_neg(ECP_FP256BN *P);
void  ECP_FP256BN_sub(ECP_FP256BN *P,ECP_FP256BN *Q);
void ECP_FP256BN_toOctet(octet *W,ECP_FP256BN *P);

int ecp_FP256BN_deserialize(ECP_FP256BN *point_out, uint8_t *buffer);
void ecp_FP256BN_serialize(uint8_t *buffer_out, ECP_FP256BN *point);
int point_on_G1_not_infinity(char * point_buffer, size_t buffer_size, ECP_FP256BN * point);

/* clang-format on */

#endif /*  EX_SSS_FIDO_ECP_H_ */
